/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package freemule.application;

/**
 * Holds availability information about the blocks of a file that is currently
 * being down- or uploaded.
 * 
 * @author <a href="mailto:droden@gmail.com">David Roden</a>
 * @version $Id: AvailabilityMap.java 130 2006-02-18 18:27:11Z bombe $
 */
public class AvailabilityMap {

	/**
	 * An array of ints, each value specifying the number of known clients who
	 * also have this block.
	 */
	protected int[] blockAvailabilities;

	/**
	 * Creates a new AvailabilityMap with the specified length.
	 * 
	 * @param length
	 *            The number of blocks
	 */
	public AvailabilityMap(int length) {
		blockAvailabilities = new int[length];
	}

	/**
	 * Sets a new availability for the specified block
	 * 
	 * @param blockIndex
	 *            The number of the block
	 * @param availability
	 *            The new availability
	 */
	public synchronized void setBlockAvailability(int blockIndex, int availability) {
		blockAvailabilities[blockIndex] = availability;
	}

	/**
	 * Returns the availability for the specified block.
	 * 
	 * @param blockIndex
	 *            The number of the block
	 * @return The number of clients that also have this block
	 */
	public synchronized int getBlockAvailability(int blockIndex) {
		return blockAvailabilities[blockIndex];
	}

	/**
	 * Increases the availability of the specified block by <code>1</code>.
	 * 
	 * @param blockIndex
	 *            The number of the block
	 */
	public synchronized void incBlockAvailability(int blockIndex) {
		blockAvailabilities[blockIndex]++;
	}

	/**
	 * Decreases the availability of the specified block by <code>1</code>.
	 * <strong>Warning:</strong> no bounds checking is performed! This method
	 * may leave you with an availability of less than zero!
	 * 
	 * @param blockIndex
	 *            The number of the block
	 */
	public synchronized void decBlockAvailability(int blockIndex) {
		blockAvailabilities[blockIndex]--;
	}

}
