/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package freemule.application;

import java.util.EventListener;

/**
 * Interface for listeners that want to be notified on client events. Client
 * events are generated when announcement information belonging to remote
 * clients has been found, or when client information times out.
 * 
 * @author <a href="mailto:droden@gmail.com">David Roden</a>
 * @version $Id: ClientListener.java 130 2006-02-18 18:27:11Z bombe $
 */
public interface ClientListener extends EventListener {

	/**
	 * Notifies listeners that the remote client has been added to the known
	 * client list.
	 * 
	 * @param remoteClient
	 *            The client to add
	 */
	public void clientAdded(Client remoteClient);

	/**
	 * Notifies listeners that the remote client was updated with more recent
	 * information.
	 * 
	 * @param remoteClient
	 *            The client to update
	 */
	public void clientUpdated(Client remoteClient);

	/**
	 * Notifies listeners that the remote client removed due to a timeout.
	 * 
	 * @param remoteClient
	 *            The client to remove
	 */
	public void clientRemoved(Client remoteClient);

}
