/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package freemule.application;

/**
 * A program module. A Service follows about the same contract as an
 * {@link java.applet.Applet}: Instantiate it, call {@link #init()}, set
 * parameters, call {@link #start()} and see it working. If you don&rsquo;t want
 * the service to run anymore, call {@link #stop()}. Afterwards call
 * {@link #destroy()} to free all resources used by the service.
 * <p>
 * The {@link AbstractService} is a convenience implementation of this
 * interface. It handles thread creation,
 * {@link freemule.application.ServiceListener service listeners}, and
 * synchronized stopping of the service and all of its threads.
 * 
 * @author David Roden &lt;droden@gmail.com&gt;
 * @version $Id: Service.java 130 2006-02-18 18:27:11Z bombe $
 */
public interface Service {

	/**
	 * Returns the name of this service.
	 * 
	 * @return The name of this service
	 */
	public String getName();

	/**
	 * Initializes this service. Has to be called before service-specified
	 * parameters are set.
	 */
	public void init();

	/**
	 * Start the service. Service-specified parameters should be set before
	 * calling this method.
	 */
	public void start();

	/**
	 * Stops this service. This method should block until the service has
	 * stopped.
	 */
	public void stop();

	/**
	 * Frees all resources used by this service.
	 */
	public void destroy();

	/**
	 * Returns whether this service is currently running.
	 * 
	 * @return <code>true</code> if this service is currently running,
	 *         <code>false</code> otherwise
	 */
	public boolean isRunning();

}
