/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package freemule.application;

/**
 * Used by the {@link DownloadService download service} to determine what to
 * download next.
 * 
 * @author <a href="mailto:droden@gmail.com">David Roden</a>
 * @version $Id: DownloadInformation.java 42 2006-02-13 16:16:40Z bombe $
 */
public class UploadInformation {

	/**
	 * The {@link FreenetFile} to download.
	 */
	private UploadFile upload;

	/**
	 * Whether to download the manifest.
	 */
	private boolean manifest;

	/**
	 * The number of the block to download.
	 */
	private int blockNumber;

	/**
	 * Creates a new DownloadInformation that specifies to download the manifest
	 * for the given download.
	 * 
	 * @param upload
	 *            The download to get the manifest for
	 */
	public UploadInformation(UploadFile upload) {
		this.upload = upload;
		this.manifest = true;
	}

	/**
	 * Creates a new DownloadInformation that specified to download the given
	 * block.
	 * 
	 * @param upload
	 *            The download to get the block for
	 * @param blockNumber
	 *            The number of the block
	 */
	public UploadInformation(UploadFile upload, int blockNumber) {
		this.upload = upload;
		this.blockNumber = blockNumber;
		this.manifest = false;
	}

	/**
	 * Returns the download.
	 * 
	 * @return The download
	 */
	public UploadFile getUploadFile() {
		return upload;
	}

	/**
	 * Returns whether this DownloadInformation specifies that the manifest
	 * should be requested.
	 * 
	 * @return <code>true</code> if the manifest should be requested,
	 *         <code>false</code>
	 */
	public boolean isManifest() {
		return manifest;
	}

	/**
	 * The number of the block to download. The return value is not defined if
	 * {@link #isManifest()} returns <code>true</code>.
	 * 
	 * @return The number of the block to download
	 */
	public int getBlockNumber() {
		return blockNumber;
	}

}
