/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package freemule.gui;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.io.File;
import java.util.HashMap;
import java.util.Map;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.Box;
import javax.swing.ButtonGroup;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.ListModel;
import javax.swing.SwingUtilities;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.border.TitledBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import freemule.main.LoggingOptions;
import freemule.main.LoggingPreferences;

/**
 * @author <a href="mailto:droden@gmail.com">David Roden</a>
 * @version $Id: LoggingPreferencesPanel.java,v 1.4 2005/12/07 01:15:20 bombe
 *          Exp $
 */
public class LoggingPreferencesPanel extends JPanel implements ActionListener, ListSelectionListener, FocusListener {

	private JFrame parent;
	private LoggingPreferences loggingPreferences;

	private JTextField logfileTextField;
	private ButtonGroup logfileButtonGroup;
	private JRadioButton logfileStdoutRadioButton;
	private JRadioButton logfileStderrRadioButton;
	private JRadioButton logfileFileRadioButton;
	private JList serviceList;
	private ListModel serviceListModel;
	protected JPanel listPanel;
	private Map<String, JPanel> servicePanels;

	private Action logfileBrowseAction;
	private Action logfileStdoutAction;
	private Action logfileStderrAction;
	private Action logfileFileAction;

	public LoggingPreferencesPanel(JFrame parent) {
		super(new BorderLayout(5, 5));
		initializeActions();
	}

	public void setLoggingPreferences(LoggingPreferences loggingPreferences) {
		this.loggingPreferences = loggingPreferences;
		removeAll();
		initializeComponents();
		initializeLayout();
	}

	private void initializeActions() {
		logfileBrowseAction = new AbstractAction("Browse") {

			public void actionPerformed(ActionEvent actionEvent) {
				actionLogfileBrowse();
			}
		};
		logfileStdoutAction = new AbstractAction("Use Standard Output") {

			public void actionPerformed(ActionEvent actionEvent) {
			}
		};
		logfileStderrAction = new AbstractAction("Use Standard Error") {

			public void actionPerformed(ActionEvent actionEvent) {
			}
		};
		logfileFileAction = new AbstractAction("Use File: ") {

			public void actionPerformed(ActionEvent actionEvent) {
			}
		};
	}

	private void initializeComponents() {
		logfileTextField = new JTextField(loggingPreferences.getLogfile());
		logfileTextField.addFocusListener(this);
		logfileButtonGroup = new ButtonGroup();
		logfileStdoutRadioButton = new JRadioButton(logfileStdoutAction);
		logfileStdoutRadioButton.addActionListener(this);
		logfileStderrRadioButton = new JRadioButton(logfileStderrAction);
		logfileStderrRadioButton.addActionListener(this);
		logfileFileRadioButton = new JRadioButton(logfileFileAction);
		logfileFileRadioButton.addActionListener(this);
		logfileButtonGroup.add(logfileStdoutRadioButton);
		logfileButtonGroup.add(logfileStderrRadioButton);
		logfileButtonGroup.add(logfileFileRadioButton);
		if ("stdout".equals(loggingPreferences.getLogfile())) {
			logfileStdoutRadioButton.doClick();
		} else if ("stderr".equals(loggingPreferences.getLogfile())) {
			logfileStderrRadioButton.doClick();
		} else {
			logfileFileRadioButton.doClick();
		}
		serviceListModel = new DefaultListModel();
		serviceList = new JList(serviceListModel);
		serviceList.addListSelectionListener(this);
		servicePanels = new HashMap<String, JPanel>();
	}

	private void initializeLayout() {
		setBorder(new EmptyBorder(5, 5, 5, 5));

		JPanel mainOptionsPanel = new JPanel(new BorderLayout(5, 5));

		Box firstLine = Box.createHorizontalBox();
		firstLine.setBorder(new CompoundBorder(new TitledBorder("Main Options"), new EmptyBorder(5, 5, 5, 5)));
		mainOptionsPanel.add(firstLine, BorderLayout.PAGE_START);
		firstLine.add(new JLabel("Logfile:"));
		firstLine.add(Box.createHorizontalStrut(5));
		firstLine.add(logfileStdoutRadioButton);
		firstLine.add(Box.createHorizontalStrut(5));
		firstLine.add(logfileStderrRadioButton);
		firstLine.add(Box.createHorizontalStrut(5));
		firstLine.add(logfileFileRadioButton);
		firstLine.add(Box.createHorizontalStrut(5));
		firstLine.add(logfileTextField);
		firstLine.add(Box.createHorizontalStrut(5));
		firstLine.add(new JButton(logfileBrowseAction));

		mainOptionsPanel.add(new JScrollPane(serviceList), BorderLayout.LINE_START);
		listPanel = new JPanel(new BorderLayout());
		mainOptionsPanel.add(new JScrollPane(listPanel), BorderLayout.CENTER);

		String[] serviceNames = loggingPreferences.getServiceNames();
		for (String serviceName: serviceNames) {
			JPanel servicePanel = new JPanel(new BorderLayout(5, 5));
			servicePanel.setBorder(new EmptyBorder(5, 5, 5, 5));
			Box optionGroupBox = Box.createVerticalBox();
			servicePanel.add(optionGroupBox, BorderLayout.PAGE_START);
			
			servicePanels.put(serviceName, servicePanel);
			((DefaultListModel) serviceListModel).addElement(serviceName);
			
			LoggingOptions serviceLoggingOptions = loggingPreferences.getServiceLoggingOptions(serviceName);
			String[] loggingGroupNames = serviceLoggingOptions.getGroups();
			for (String loggingGroupName: loggingGroupNames) {
				Box optionGroupPanel = Box.createVerticalBox();
				optionGroupPanel.setBorder(new CompoundBorder(new TitledBorder(loggingGroupName), new EmptyBorder(0, 5, 5, 5)));
				optionGroupBox.add(optionGroupPanel);
				optionGroupBox.add(Box.createVerticalStrut(5));
				String[] loggingGroupEventNames = serviceLoggingOptions.getGroupEventNames(loggingGroupName);
				for (String loggingGroupEventName: loggingGroupEventNames) {
					Box loggingGroupEventBox = Box.createHorizontalBox();
					optionGroupPanel.add(loggingGroupEventBox);
					loggingGroupEventBox.add(new JLabel(loggingGroupEventName));
					loggingGroupEventBox.add(Box.createHorizontalGlue());
					JCheckBox logToGuiCheckBox = new JCheckBox("Log to GUI");
					logToGuiCheckBox.addActionListener(this);
					logToGuiCheckBox.putClientProperty("Service Name", serviceName);
					logToGuiCheckBox.putClientProperty("Event Name", loggingGroupEventName);
					logToGuiCheckBox.putClientProperty("Log Channel", "GUI");
					logToGuiCheckBox.setSelected(loggingPreferences.getGUILogging(serviceName, loggingGroupEventName));
					JCheckBox logToFileCheckBox = new JCheckBox("Log to File");
					logToFileCheckBox.putClientProperty("Service Name", serviceName);
					logToFileCheckBox.putClientProperty("Event Name", loggingGroupEventName);
					logToFileCheckBox.putClientProperty("Log Channel", "File");
					logToFileCheckBox.setSelected(loggingPreferences.getFileLogging(serviceName, loggingGroupEventName));
					loggingGroupEventBox.add(logToGuiCheckBox);
					loggingGroupEventBox.add(Box.createHorizontalStrut(5));
					loggingGroupEventBox.add(logToFileCheckBox);
				}
			}
			optionGroupBox.add(Box.createVerticalGlue());
		}

		add(mainOptionsPanel, BorderLayout.CENTER);
	}

	protected void actionLogfileBrowse() {
		JFileChooser fileChooser = new JFileChooser();
		fileChooser.showSaveDialog(parent);
		File logfileFile = fileChooser.getSelectedFile();
		if (logfileFile != null) {
			logfileTextField.setText(logfileFile.getAbsolutePath());
		}
	}

	public void actionPerformed(ActionEvent actionEvent) {
		Object source = actionEvent.getSource();
		if (source instanceof JCheckBox) {
			JCheckBox sourceCheckBox = (JCheckBox) source;
			String serviceName = (String) sourceCheckBox.getClientProperty("Service Name");
			String eventName = (String) sourceCheckBox.getClientProperty("Event Name");
			String logChannel = (String) sourceCheckBox.getClientProperty("Log Channel");
			LoggingOptions serviceLoggingOptions = loggingPreferences.getServiceLoggingOptions(serviceName);
			if (logChannel.equals("GUI")) {
				serviceLoggingOptions.setGUILogging(eventName, sourceCheckBox.isSelected());
			} else if (logChannel.equals("File")) {
				serviceLoggingOptions.setFileLogging(eventName, sourceCheckBox.isSelected());
			}
		} else if (source instanceof JRadioButton) {
			JRadioButton sourceRadioButton = (JRadioButton) source;
			logfileBrowseAction.setEnabled(sourceRadioButton == logfileFileRadioButton);
			logfileTextField.setEnabled(sourceRadioButton == logfileFileRadioButton);
			if (sourceRadioButton == logfileStdoutRadioButton) {
				loggingPreferences.setLogfile("stdout");
			} else if (sourceRadioButton == logfileStderrRadioButton) {
				loggingPreferences.setLogfile("stderr");
			} else if (sourceRadioButton == logfileFileRadioButton) {
				loggingPreferences.setLogfile(logfileTextField.getText().trim());
			}
		}
	}

	/**
	 * {@inheritDoc}
	 */
	public void valueChanged(ListSelectionEvent listSelectionEvent) {
		int selectedRow = serviceList.getSelectedIndex();
		if (selectedRow != -1) {
			String serviceName = (String) serviceListModel.getElementAt(selectedRow);
			final JPanel servicePanel = servicePanels.get(serviceName);
			SwingUtilities.invokeLater(new Runnable() {
				public void run() {
					listPanel.removeAll();
					listPanel.add(servicePanel, BorderLayout.CENTER);
					listPanel.revalidate();
				}
			});
		}
	}

	/**
	 * {@inheritDoc}
	 */
	public void focusGained(FocusEvent e) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void focusLost(FocusEvent e) {
		Object source = e.getSource();
		if (source == logfileTextField) {
			loggingPreferences.setLogfile(logfileTextField.getText().trim());
		}
	}

}
