/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package freemule.gui;

import javax.swing.table.AbstractTableModel;

import freemule.application.Client;
import freemule.application.SharedFile;

/**
 * @author <a href="mailto:droden@gmail.com">David Roden</a>
 * @version $Id: SearchResultCommentTableModel.java,v 1.2 2005/12/03 11:05:16
 *          bombe Exp $
 */
public class SearchResultCommentTableModel extends AbstractTableModel {

	private static final String[] COLUMN_NAMES = new String[] { "Client", "Rating", "Comment" };
	private static final Class[] COLUMN_CLASSES = new Class[] { String.class, String.class, String.class };

	private Client[] clients;
	private String[] comments;
	private int[] ratings;

	public SearchResultCommentTableModel(Client[] clients, String[] comments, int[] ratings) {
		this.clients = clients;
		this.comments = comments;
		this.ratings = ratings;
	}

	/**
	 * {@inheritDoc}
	 */
	public int getRowCount() {
		return comments.length;
	}

	/**
	 * {@inheritDoc}
	 */
	public int getColumnCount() {
		return COLUMN_NAMES.length;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String getColumnName(int column) {
		return COLUMN_NAMES[column];
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Class< ? > getColumnClass(int columnIndex) {
		return COLUMN_CLASSES[columnIndex];
	}

	/**
	 * {@inheritDoc}
	 */
	public Object getValueAt(int rowIndex, int columnIndex) {
		switch (columnIndex) {
		case 0:
			return clients[rowIndex].getName();
		case 1:
			return SharedFile.RATING[ratings[rowIndex]];
		case 2:
			return comments[rowIndex];
		}
		return null;
	}

}
