/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package freemule.gui;

import javax.swing.table.AbstractTableModel;

import freemule.application.SharedFile;
import freemule.application.SharedFileListener;
import freemule.application.SharedFileService;

/**
 * @author <a href="mailto:droden@gmail.com">David Roden</a>
 * @version $Id: SharedFileTableModel.java 130 2006-02-18 18:27:11Z bombe $
 */
public class SharedFileTableModel extends AbstractTableModel implements SharedFileListener {

	private static final String[] columnNames = { "Filename", "Length", "Comment", "Rating", "Key", "Requests", "Uploads" };
	private static final Class< ? >[] columnClasses = { String.class, Long.class, String.class, String.class, String.class, Integer.class, Integer.class };

	private SharedFileService sharedFileService;
	private SharedFile[] sharedFiles;

	private void updateSharedFiles() {
		if (sharedFileService == null) {
			sharedFiles = null;
			return;
		}
		sharedFiles = sharedFileService.getSharedFiles();
		fireTableDataChanged();
	}

	public void setSharedFileService(SharedFileService sharedFileService) {
		if (this.sharedFileService != null) {
			this.sharedFileService.removeSharedFileListener(this);
		}
		this.sharedFileService = sharedFileService;
		if (this.sharedFileService != null) {
			this.sharedFileService.addSharedFileListener(this);
		}
		updateSharedFiles();
	}

	public SharedFile[] getSharedFiles() {
		return sharedFiles;
	}

	/**
	 * @see javax.swing.table.TableModel#getRowCount()
	 */
	public int getRowCount() {
		if (sharedFiles == null) {
			return 0;
		}
		return sharedFiles.length;
	}

	/**
	 * @see javax.swing.table.TableModel#getColumnCount()
	 */
	public int getColumnCount() {
		return columnNames.length;
	}

	/**
	 * @see javax.swing.table.TableModel#getValueAt(int, int)
	 */
	public Object getValueAt(int rowIndex, int columnIndex) {
		if (sharedFiles == null) {
			return null;
		}
		SharedFile sharedFile = sharedFiles[rowIndex];
		if (columnIndex == -1) {
			return sharedFile;
		} else if (columnIndex == 0) {
			return sharedFile.getName();
		} else if (columnIndex == 1) {
			return new Long(sharedFile.getLength());
		} else if (columnIndex == 2) {
			return sharedFile.getComment();
		} else if (columnIndex == 3) {
			return SharedFile.RATING[sharedFile.getRating()];
		} else if (columnIndex == 4) {
			return sharedFile.getKey();
		} else if (columnIndex == 5) {
			return new Integer(sharedFile.getRequests());
		} else if (columnIndex == 6) {
			return new Integer(sharedFile.getUploads());
		}
		return null;
	}

	/**
	 * @see javax.swing.table.AbstractTableModel#getColumnName(int)
	 */
	@Override
	public String getColumnName(int column) {
		return columnNames[column];
	}

	/**
	 * @see javax.swing.table.AbstractTableModel#getColumnClass(int)
	 */
	@Override
	public Class< ? > getColumnClass(int columnIndex) {
		return columnClasses[columnIndex];
	}

	/**
	 * {@inheritDoc}
	 */
	public void sharedFileLoadingStarted(String filename) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void sharedFileLoadingFailed(String filename, Exception exception) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void sharedFileLoadingComplete(String filename) {
		updateSharedFiles();
	}

	/**
	 * {@inheritDoc}
	 */
	public void sharedFileUnsharedFile(String filename) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void sharedFileUnsharedDirectory(String directory) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void sharedFileSavingStarted(String filename) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void sharedFileSavingFailed(String filename, Exception exception) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void sharedFileSavingComplete(String filename) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void sharedFileScanStarted() {
	}

	/**
	 * {@inheritDoc}
	 */
	public void sharedFileScanFailed(Exception exception) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void sharedFileScanComplete() {
		updateSharedFiles();
	}

	/**
	 * {@inheritDoc}
	 */
	public void sharedFileDirectoryScanStarted(String directory) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void sharedFileDirectoryScanFailed(String directory, Exception exception) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void sharedFileDirectoryScanComplete(String directory) {
		updateSharedFiles();
	}

	/**
	 * {@inheritDoc}
	 */
	public void sharedFileHashingStarted(String filename) {
	}
	
	/**
	 * {@inheritDoc}
	 */
	public void sharedFileHashingProgress(String filename, int blockIndex, int blockCount) {
		
	}

	/**
	 * {@inheritDoc}
	 */
	public void sharedFileHashingFailed(String filename, Exception exception) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void sharedFileHashingComplete(String filename, String key, long filelength, long duration) {
		updateSharedFiles();
	}

	/**
	 * {@inheritDoc}
	 */
	public void sharedFileListChanged() {
		updateSharedFiles();
	}

}
