/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package freemule.gui;

import javax.swing.table.AbstractTableModel;

import freemule.application.FreenetFile;
import freemule.application.UploadFile;
import freemule.application.UploadListener;
import freemule.application.UploadService;

/**
 * @author <a href="mailto:droden@gmail.com">David Roden</a>
 * @version $Id: UploadFilesTableModel.java 130 2006-02-18 18:27:11Z bombe $
 */
public class UploadFilesTableModel extends AbstractTableModel implements UploadListener {

	private static final String[] COLUMN_NAMES = new String[] { "Name", "Length", "Progress" };
	private static final Class[] COLUMN_CLASSES = new Class[] { String.class, Long.class, FreenetFile.class };

	private UploadFile[] uploadFiles;
	private UploadService uploadService;

	public UploadFilesTableModel() {
		uploadFiles = new UploadFile[0];
	}

	public void setUploadService(UploadService uploadService) {
		if (this.uploadService != null) {
			this.uploadService.removeUploadListener(this);
		}
		this.uploadService = uploadService;
		if (this.uploadService != null) {
			this.uploadService.addUploadListener(this);
		}
		updateUploads();
	}

	private void updateUploads() {
		uploadFiles = uploadService.getUploadFiles();
		fireTableDataChanged();
	}

	//
	// INTERFACE TableModel
	//

	/**
	 * {@inheritDoc}
	 */
	public int getRowCount() {
		return uploadFiles.length;
	}

	/**
	 * {@inheritDoc}
	 */
	public int getColumnCount() {
		return COLUMN_NAMES.length;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Class< ? > getColumnClass(int columnIndex) {
		return COLUMN_CLASSES[columnIndex];
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String getColumnName(int column) {
		return COLUMN_NAMES[column];
	}

	/**
	 * {@inheritDoc}
	 */
	public Object getValueAt(int rowIndex, int columnIndex) {
		UploadFile uploadFile = uploadFiles[rowIndex];
		switch (columnIndex) {
		case -1:
			return uploadFile;
		case 0:
			return uploadFile.getName();
		case 1:
			return new Long(uploadFile.getLength());
		case 2:
			return uploadFile;
		}
		return null;
	}

	private int indexOf(UploadFile uploadFile) {
		for (int index = 0, count = uploadFiles.length; index < count; index++) {
			if (uploadFiles[index].equals(uploadFile)) {
				return index;
			}
		}
		return -1;
	}

	//
	// INTERFACE UploadListener
	//

	/**
	 * {@inheritDoc}
	 */
	public void uploadAdded(UploadFile uploadFile) {
		updateUploads();
	}

	/**
	 * {@inheritDoc}
	 */
	public void uploadUpdated(UploadFile uploadFile) {
		updateUploads();
	}

	/**
	 * {@inheritDoc}
	 */
	public void uploadComplete(UploadFile uploadFile) {
		updateUploads();
	}

	/**
	 * {@inheritDoc}
	 */
	public void uploadRemoved(UploadFile uploadFile) {
		updateUploads();
	}

	/**
	 * {@inheritDoc}
	 */
	public void blockUploadStarted(UploadFile uploadFile, int blockIndex) {
		int row = indexOf(uploadFile);
		fireTableRowsUpdated(row, row);
	}

	/**
	 * {@inheritDoc}
	 */
	public void blockUploadFailed(UploadFile uploadFile, int blockIndex, Exception exception) {
		int row = indexOf(uploadFile);
		fireTableRowsUpdated(row, row);
	}

	/**
	 * {@inheritDoc}
	 */
	public void blockUploadComplete(UploadFile uploadFile, int blockIndex) {
		int row = indexOf(uploadFile);
		fireTableRowsUpdated(row, row);
	}

	/**
	 * {@inheritDoc}
	 */
	public void manifestUploadStarted(UploadFile uploadFile) {
		int row = indexOf(uploadFile);
		fireTableRowsUpdated(row, row);
	}

	/**
	 * {@inheritDoc}
	 */
	public void manifestUploadFailed(UploadFile uploadFile, Exception exception) {
		int row = indexOf(uploadFile);
		fireTableRowsUpdated(row, row);
	}

	/**
	 * {@inheritDoc}
	 */
	public void manifestUploadComplete(UploadFile uploadFile) {
		int row = indexOf(uploadFile);
		fireTableRowsUpdated(row, row);
	}

}
