/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package freemule.splash;

import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.image.ImageObserver;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;


/**
 * @author David Roden &lt;droden@gmail.com&gt;
 * @version $Id: Splash.java 130 2006-02-18 18:27:11Z bombe $
 * @license GPL
 */
public class Splash extends Dialog implements ImageObserver {

	private String imageResourceName;
	private Image splashImage;
	private int imageWidth;
	private int imageHeight;
	
	public Splash(String imageResourceName) {
		super(new Frame());
		setUndecorated(true);
		this.imageResourceName = imageResourceName;
		if (loadImage()) {
			showImage();
		}
	}

	private boolean loadImage() {
		try {
			InputStream resourceStream = getClass().getResourceAsStream(imageResourceName);
			ByteArrayOutputStream byteOutput = new ByteArrayOutputStream(131072);
			int r = 0;
			while (r != -1) {
				byte[] buffer = new byte[131072];
				r = resourceStream.read(buffer);
				if (r > -1) {
					byteOutput.write(buffer);
				}
			}
			byteOutput.close();
			resourceStream.close();
			byte[] imageBytes = byteOutput.toByteArray();
			splashImage = Toolkit.getDefaultToolkit().createImage(imageBytes);
			return true;
		} catch (Throwable t) {
		}
		return false;
	}
	
	private void showImage() {
		splashImage.getWidth(this);
		splashImage.getHeight(this);
	}
	
	private void setCorrectSize() {
		Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
		setSize(imageWidth, imageHeight);
		setLocation((screen.width - imageWidth) / 2, (screen.height - imageHeight) / 2);
		setAlwaysOnTop(true);
		setVisible(true);
	}
	
	public void paint(Graphics graphics) {
		if ((imageWidth != 0) && (imageHeight != 0)) {
			graphics.drawImage(splashImage, 0, 0, this);
		}
	}
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean imageUpdate(Image img, int infoflags, int x, int y, int width, int height) {
		if ((infoflags & WIDTH) != 0) {
			this.imageWidth = width;
		}
		if ((infoflags & HEIGHT) != 0) {
			this.imageHeight = height;
		}
		if ((imageWidth != 0) && (imageHeight != 0)) {
			setCorrectSize();
		}
		return super.imageUpdate(img, infoflags, x, y, width, height);
	}
	
	/**
	 * @param visible
	 * @param delay The delay in milliseconds
	 */
	public void setVisible(boolean visible, final long delay) {
		if (!visible) {
			new Thread(new Runnable() {
				public void run() {
					try {
						Thread.sleep(delay);
					} catch (InterruptedException e) {
					}
					setVisible(false);
				}
			}).start();
		} else {
			setVisible(visible);
		}
	}

}
