/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package freemule.util;

import java.util.Calendar;

/**
 * Helper class that formats the date of a {@link java.util.Calendar} in a way
 * that is used by the {@link freemule.application FreeMule application}.
 * 
 * @author <a href="mailto:droden@gmail.com">David Roden</a>
 * @version $Id: DateFormatter.java 130 2006-02-18 18:27:11Z bombe $
 */
public class DateFormatter {

	/**
	 * Zero-pads a number up to the specified number of digits.
	 * 
	 * @param number
	 *            The number to zero-pad
	 * @param digits
	 *            The number of digits for the number
	 * @return The zero-padded number
	 */
	public static String nullify(int number, int digits) {
		String text = String.valueOf(number);
		while (text.length() < digits) {
			text = "0" + text;
		}
		return text;
	}

	/**
	 * Converts the current hour of <code>calendar</code> into a path name
	 * portion used for e.g. announcement or upload request inserts.
	 * 
	 * @param calendar
	 *            The calendar to format
	 * @return A path name portion in the form of
	 *         <code>"<i>yyyy</i>/<i>mm</i>/<i>dd</i>/<i>hh</i>"</code>
	 */
	public static String getKeyPortion(Calendar calendar) {
		return nullify(calendar.get(Calendar.YEAR), 4) + "/" + nullify(calendar.get(Calendar.MONTH) + 1, 2) + "/" + nullify(calendar.get(Calendar.DAY_OF_MONTH), 2) + "/" + nullify(calendar.get(Calendar.HOUR_OF_DAY), 2);
	}

}
