/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package freenet.fcp;

import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.List;


/**
 * @author David Roden <droden@gmail.com>
 * @version $Id: FcpFecMakeMetadata.java,v 1.1 2005/11/28 22:54:44 bombe Exp $
 */
public class FcpFecMakeMetadata extends FcpCommand {

	private String description;
	private String mimeType;
	private List/*<SegmentHeader>*/ segmentHeaders;
	private List/*<BlockMap>*/ blockMaps;
	private byte[] metadata;
	private int dataPointer;

	public FcpFecMakeMetadata(String description, String mimeType, SegmentHeader[] segmentHeaders, BlockMap[] blockMaps) {
		this.description = description;
		this.mimeType = mimeType;
		this.segmentHeaders = new ArrayList();
		for (int index = 0, size = segmentHeaders.length; index < size; index++) {
			this.segmentHeaders.add(segmentHeaders[index]);
		}
		this.blockMaps = new ArrayList();
		for (int index = 0, size = blockMaps.length; index < size; index++) {
			this.blockMaps.add(blockMaps[index]);
		}
	}

	public FcpFecMakeMetadata(String description, String mimeType, List/*<SegmentHeader>*/ segmentHeaders, List/*<BlockMap>*/ blockMaps) {
		this.description = description;
		this.mimeType = mimeType;
		this.segmentHeaders = segmentHeaders;
		this.blockMaps = blockMaps;
	}
	
	/**
	 * @see freenet.fcp.FcpCommand#getName()
	 */
	public String getName() {
		return "FECMakeMetadata";
	}

	public byte[] getMetadata() {
		return metadata;
	}
	
	/**
	 * @see freenet.fcp.FcpCommand#writeCommand(java.io.OutputStream)
	 */
	public boolean writeCommand(OutputStream outputStream) throws IOException {
		if ((description != null) && (description.length() > 0)) {
			outputStream.write(("Description=" + description + "\r\n").getBytes());
		} else {
			outputStream.write(("Description=\r\n").getBytes());
		}
		if ((mimeType != null) && (mimeType.length() > 0)) {
			outputStream.write(("MimeType=" + mimeType + "\r\n").getBytes());
		} else {
			outputStream.write(("MimeType=\r\n").getBytes());
		}
		long dataLength = 0;
		for (int segmentIndex = 0, segmentCount = segmentHeaders.size(); segmentIndex < segmentCount; segmentIndex++) {
			dataLength += ((SegmentHeader) segmentHeaders.get(segmentIndex)).toByteArray().length;
			dataLength += ((BlockMap) blockMaps.get(segmentIndex)).toByteArray().length;
		}
		outputStream.write(("DataLength=" + encodeHex(dataLength) + "\r\n").getBytes());
		outputStream.write("Data\r\n".getBytes());
		for (int segmentIndex = 0, segmentCount = segmentHeaders.size(); segmentIndex < segmentCount; segmentIndex++) {
			outputStream.write(((SegmentHeader) segmentHeaders.get(segmentIndex)).toByteArray());
			outputStream.write(((BlockMap) blockMaps.get(segmentIndex)).toByteArray());
		}
		return false;
	}

	/**
	 * @see freenet.fcp.FcpCommand#handleFcpMessage(freenet.fcp.FcpMessage)
	 */
	public void handleFcpMessage(FcpMessage fcpMessage) throws FcpException {
		String fcpMessageName = fcpMessage.getName();
		if (fcpMessageName.equals("MadeMetadata")) {
			metadata = new byte[(int) parseHex(fcpMessage.getParameter("DataLength"))];
			dataPointer = 0;
		} else if (fcpMessageName.equals("Restarted")) {
			dataPointer = 0;
		} else if (fcpMessageName.equals("DataChunk")) {
			int length = (int) parseHex(fcpMessage.getParameter("Length"));
			System.arraycopy(fcpMessage.getData(), 0, metadata, dataPointer, length);
			dataPointer += length;
			complete = dataPointer == metadata.length;
		}
	}

}
