/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package freenet.fcp;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;

public class SegmentHeader {

	private static final String CRLF = "\r\n";
	private String fecAlgorithm;
	private long fileLength;
	private long offset;
	private int blockCount;
	private int blockSize;
	private int checkBlockCount;
	private int checkBlockSize;
	private int segmentCount;
	private int segment;
	private int blocksRequired;

	/**
	 * @return Returns the blockCount.
	 */
	public int getBlockCount() {
		return blockCount;
	}

	/**
	 * @param blockCount
	 *                  The blockCount to set.
	 */
	public void setBlockCount(int blockCount) {
		this.blockCount = blockCount;
	}

	/**
	 * @return Returns the blockSize.
	 */
	public int getBlockSize() {
		return blockSize;
	}

	/**
	 * @param blockSize
	 *                  The blockSize to set.
	 */
	public void setBlockSize(int blockSize) {
		this.blockSize = blockSize;
	}

	/**
	 * @return Returns the blocksRequired.
	 */
	public int getBlocksRequired() {
		return blocksRequired;
	}

	/**
	 * @param blocksRequired
	 *                  The blocksRequired to set.
	 */
	public void setBlocksRequired(int blocksRequired) {
		this.blocksRequired = blocksRequired;
	}

	/**
	 * @return Returns the checkBlockCount.
	 */
	public int getCheckBlockCount() {
		return checkBlockCount;
	}

	/**
	 * @param checkBlockCount
	 *                  The checkBlockCount to set.
	 */
	public void setCheckBlockCount(int checkBlockCount) {
		this.checkBlockCount = checkBlockCount;
	}

	/**
	 * @return Returns the checkBlockSize.
	 */
	public int getCheckBlockSize() {
		return checkBlockSize;
	}

	/**
	 * @param checkBlockSize
	 *                  The checkBlockSize to set.
	 */
	public void setCheckBlockSize(int checkBlockSize) {
		this.checkBlockSize = checkBlockSize;
	}

	/**
	 * @return Returns the fecAlgorithm.
	 */
	public String getFecAlgorithm() {
		return fecAlgorithm;
	}

	/**
	 * @param fecAlgorithm
	 *                  The fecAlgorithm to set.
	 */
	public void setFecAlgorithm(String fecAlgorithm) {
		this.fecAlgorithm = fecAlgorithm;
	}

	/**
	 * @return Returns the fileLength.
	 */
	public long getFileLength() {
		return fileLength;
	}

	/**
	 * @param fileLength
	 *                  The fileLength to set.
	 */
	public void setFileLength(long fileLength) {
		this.fileLength = fileLength;
	}

	/**
	 * @return Returns the offset.
	 */
	public long getOffset() {
		return offset;
	}

	/**
	 * @param offset
	 *                  The offset to set.
	 */
	public void setOffset(long offset) {
		this.offset = offset;
	}

	/**
	 * @return Returns the segment.
	 */
	public int getSegment() {
		return segment;
	}

	/**
	 * @param segment
	 *                  The segment to set.
	 */
	public void setSegment(int segment) {
		this.segment = segment;
	}

	/**
	 * @return Returns the segmentCount.
	 */
	public int getSegmentCount() {
		return segmentCount;
	}

	/**
	 * @param segmentCount
	 *                  The segmentCount to set.
	 */
	public void setSegmentCount(int segmentCount) {
		this.segmentCount = segmentCount;
	}

	public byte[] toByteArray() {
		ByteArrayOutputStream byteOutput = new ByteArrayOutputStream();
		PrintWriter writer = new PrintWriter(new OutputStreamWriter(byteOutput));
		writer.print("SegmentHeader" + CRLF);
		writer.print("FECAlgoName=" + fecAlgorithm + CRLF);
		writer.print("FileLength=" + FcpCommand.encodeHex(fileLength) + CRLF);
		writer.print("Offset=" + FcpCommand.encodeHex(offset) + CRLF);
		writer.print("BlockCount=" + FcpCommand.encodeHex(blockCount) + CRLF);
		writer.print("BlockSize=" + FcpCommand.encodeHex(blockSize) + CRLF);
		writer.print("CheckBlockCount=" + FcpCommand.encodeHex(checkBlockCount) + CRLF);
		writer.print("CheckBlockSize=" + FcpCommand.encodeHex(checkBlockSize) + CRLF);
		writer.print("Segments=" + FcpCommand.encodeHex(segmentCount) + CRLF);
		writer.print("SegmentNum=" + FcpCommand.encodeHex(segment) + CRLF);
		writer.print("BlocksRequired=" + FcpCommand.encodeHex(blocksRequired) + CRLF);
		writer.print("EndMessage" + CRLF);
		writer.close();
		try {
			byteOutput.close();
		} catch (IOException ioe1) {
		}
		return byteOutput.toByteArray();
	}

}